#!/usr/bin/env python
# -*- coding:utf-8 -*-

import json, requests, socket

class Client():
    def __init__(self, protocol = 'http', host='localhost', port='8888'):
        self._protocol = protocol
        self._host = host
        self._port = port
        self._baseurl = "{}://{}:{}".format(self._protocol, self._host, self._port)
        self._db=None
        self._timeout = 10
        self._header = {'Content-type': 'application/json'}
        self.check_networking()

    def check_networking(self):
        socket.setdefaulttimeout(self._timeout)
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        check_result = s.connect_ex((self._host, int(self._port)))
        s.close()
        if check_result == 0:
            print("[INFO] Connected to {}:{} successfully.".format(self._host, self._port))
        else:
            raise Exception("[ERROR] Failed to connect to {}:{}".format(self._host, self._port))

    def welcome(self):
        req_url = "{}/".format(self._baseurl)
        req_data= None
        res = requests.get(url=req_url, data=json.dumps(req_data), headers=self._header, timeout=self._timeout)
        status_code = res.status_code
        body = res.text
        return status_code, body

    def state(self):
        req_url = "{}/state".format(self._baseurl)
        req_data= None
        res = requests.get(url=req_url, data=json.dumps(req_data), headers=self._header)
        status_code = res.status_code
        body = res.json()
        return status_code, body

    def use_db(self, db_name):
        self._db = db_name

    def load_db(self, db_name, db_path, vector_scale=None):
        req_url = "{}/api/load".format(self._baseurl)
        req_data= {"name": db_name, "path": db_path}
        if vector_scale is not None:
            req_data["vectorScale"] = vector_scale
        res = requests.post(url=req_url, data=json.dumps(req_data), headers=self._header)
        status_code = res.status_code
        body = res.json()
        return status_code, body

    def unload_db(self, db_name):
        req_url = "{}/api/{}/unload".format(self._baseurl, db_name)
        res = requests.post(url=req_url, data=None, headers=self._header)
        status_code = res.status_code
        body = res.json()
        return status_code, body

    def create_table(self, table_name="MyTable", table_fields=[]):
        if self._db is None:
            raise Exception("[ERROR] Please use_db() first!")
        req_url = "{}/api/{}/schema/tables".format(self._baseurl, self._db)
        req_data= {"name": table_name, "fields": table_fields}
        res = requests.post(url=req_url, data=json.dumps(req_data), headers=self._header)
        status_code = res.status_code
        body = res.json()
        return status_code, body


    def insert(self, table_name="MyTable", records=[]):
        if self._db is None:
            raise Exception("[ERROR] Please use_db() first!")
        req_url = "{}/api/{}/data/insert".format(self._baseurl, self._db)
        req_data= {"table": table_name, "data": records}
        res = requests.post(url=req_url, data=json.dumps(req_data), headers=self._header)
        status_code = res.status_code
        body = res.json()
        return status_code, body


    def query(self, table_name="MyTable", query_field="", query_vector=[], response_fields=[], limit=1):
        if self._db is None:
            raise Exception("[ERROR] Please use_db() first!")
        req_url = "{}/api/{}/data/query".format(self._baseurl, self._db)
        req_data= {"table": table_name, "queryField": query_field, "queryVector": query_vector, "response": response_fields, "limit": limit}
        res = requests.post(url=req_url, data=json.dumps(req_data), headers=self._header)
        status_code = res.status_code
        body = res.json()
        return status_code, body


    def drop_table(self, table_name="MyTable"):
        if self._db is None:
            raise Exception("[ERROR] Please use_db() first!")
        req_url = "{}/api/{}/schema/tables/{}".format(self._baseurl, self._db, table_name)
        req_data= None
        res = requests.delete(url=req_url, data=json.dumps(req_data), headers=self._header)
        status_code = res.status_code
        body = res.json()
        return status_code, body

    def drop_db(self, db_name):
        req_url = "{}/api/{}/drop".format(self._baseurl, db_name)
        res = requests.delete(url=req_url, data=None, headers=self._header)
        status_code = res.status_code
        body = res.json()
        return status_code, body



#!/usr/bin/env python
# -*- coding:utf-8 -*-

# Try this simple example
# 1. docker run --pull=always -d -p 8888:8888 epsilla/vectordb
# 2. pip3 install --upgrade pyepsilla
# 3. python3 simple_example.py
#

from pyepsilla import vectordb
import random, string, time

## Connect to Epsilla VectorDB
c = vectordb.Client(host='127.0.0.1', port='8888')
# c = vectordb.Client(host='3.209.6.179', port='8888', db_name='default')

## Check VectorDB Status
status_code, response = c.welcome()
status_code, response = c.state()

## Load DB with path
status_code, response= c.load_db(db_name="myDB", db_path="/tmp/epsilla")

## Set DB to current DB
c.use_db(db_name="myDB")

## Unload DB
# c.unload(db_name="myDB")


## Create a table with schema in current DB
### define records number and vector dimension
records_num = 3000
dimensions = 8

id_field = {"name": "ID", "dataType": "INT"}
doc_field = {"name": "Doc", "dataType": "STRING"}
vec_field = {"name": "Embedding", "dataType": "VECTOR_FLOAT", "dimensions": dimensions}

fields = [id_field, doc_field, vec_field]
status_code, response = c.create_table(table_name="MyTable", table_fields=fields)


## Insert new vector records into table
# Ids = [ i for i in range(5)]
# Docs = ["Berlin", "London", "Moscow", "San Francisco", "Shanghai"]
# Embedding =[[0.05, 0.61, 0.76, 0.74],
#        [0.19, 0.81, 0.75, 0.11],
#        [0.36, 0.55, 0.47, 0.94],
#        [0.18, 0.01, 0.85, 0.80],
#        [0.24, 0.18, 0.22, 0.44]
#       ]

## Insert new vector records
letters = list(string.ascii_lowercase+string.ascii_uppercase+string.digits)
Docs = [''.join(random.choices(letters, k=6)) for _ in range(records_num)]
Embedding = [[random.random() for _ in range(dimensions)] for _ in range(records_num)]

records_data = [ {"ID": i, "Doc": Docs[i], "Embedding": Embedding[i]} for i in range(records_num)]
status_code, response = c.insert(table_name="MyTable", records=records_data)


## Query Vectors
query_field = "Embedding"
query_vector = Embedding[-1]
print(Docs[-1], query_vector)
response_fields = ["Doc"]
limit = 2
status_code, response = c.query(table_name="MyTable", query_field=query_field, query_vector=query_vector, response_fields=response_fields, limit=limit)
print("status_code", status_code, "response", response)

## Drop table
#status_code, response = c.drop_table("MyTable")

## Drop db
#status_code, response = c.drop_db("myDB")

